import QtQuick 1.1
import Qt.labs.folderlistmodel 1.0
import com.nokia.meego 1.0

PageStackWindow
{
  id: rootWindow

  // This is what we see when the app starts, it links to the component specific pages
  initialPage: mode==3? messagePage:mode==2? aboutPage:mode==1? filePage:mainPage

  //
  // These tools are shared by most sub-pages by assigning the id to a page's tools property
  //
  ToolBarLayout
  {
    id: commonTools
    visible: false
    ToolIcon { iconId: "toolbar-back"; onClicked: { myMenu.close(); if(pageStack.depth>1) pageStack.pop(); else Qt.quit(); } }
    ToolIcon { iconId: "toolbar-view-menu"; onClicked: (myMenu.status == DialogStatus.Closed) ? myMenu.open() : myMenu.close() }
  }

  Menu
  {
    id: myMenu

    // visualParent is needed to specify the grayed out area.
    visualParent: pageStack

    MenuLayout { }
  }

  //
  // This is the main page that gets shown on startup
  //
  Page
  {
    id: mainPage
    anchors.margins: 0
    tools: commonTools

    ScrollDecorator { flickableItem: flickable1 }

    Flickable
    {
      id: flickable1
      anchors.fill: parent
      flickableDirection: Flickable.VerticalFlick
      contentHeight: column1.height

      Column
      {
        id: column1
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: 5
        width: parent.width-30

        Label
        {
          text: emulib.title
          width: parent.width
          height: font.pixelSize+16
          horizontalAlignment: Text.AlignHCenter
          verticalAlignment: Text.AlignBottom
          font.bold: true
          font.pixelSize: 50
          color: "#800000"
        }

        Label
        {
          text: "by "+emulib.author
          width: parent.width
          height: font.pixelSize+16
          horizontalAlignment: Text.AlignHCenter
          verticalAlignment: Text.AlignVCenter
          font.bold: true
          font.pixelSize: 24
        }

        Button
        { 
          id:    btnOpenFile;
          width: parent.width
          text:  "Open file"
          onClicked: pageStack.push(filePage)
        }

        Button
        { 
          id:    btnFreeze;
          width: parent.width
          text:  "Freeze state"
          onClicked: { emulib.command="freeze";Qt.quit(); }
        }

        Button
        { 
          id:    btnRestore;
          width: parent.width
          text:  "Restore state"
          onClicked: { emulib.command="restore";Qt.quit(); }
        }

        Button
        {
          id:    btnAppConfig
          text:  emulib.title+" settings"
          width: parent.width
          onClicked: { emulib.command="menu";Qt.quit(); }     
        }

        Button
        {
          id:    btnMeegoConfig
          text:  "Meego settings"
          width: parent.width
          onClicked: pageStack.push(configPage)
        }

        Button
        {
          id:    btnNetPlay
          text:  "Network play"
          width: parent.width
          onClicked: { emulib.command="netplay";Qt.quit(); }
        }

        Button
        {
          id:    btnDebugger
          text:  "Debugger"
          width: parent.width
          onClicked: { emulib.command="debug";Qt.quit(); }
        }

        Button
        {
          id:    btnAbout
          text:  "About "+emulib.title
          width: parent.width
          onClicked: pageStack.push(aboutPage)
        }

        Button
        {
          id:    btnReset
          text:  "Reset emulation"
          width: parent.width
          onClicked: { emulib.command="reset";Qt.quit(); }
        }

        Button
        {
          id:    btnQuit
          text:  "Quit application"
          width: parent.width
          onClicked: { emulib.command="quit";Qt.quit(); }
        }
      }
    }
  }

  //
  // This is the Settings page
  //
  Page
  {
    id: configPage
    anchors.margins: 0
    tools: commonTools

    Column
    {
      anchors.horizontalCenter: parent.horizontalCenter
      spacing: 5
      width: parent.width
      height: parent.height

      Rectangle
      {
        id: configTop
        height: 64
        width: parent.width
        color: "#40A0FF"
        z: 10

        Label
        {
          text: emulib.title+" Settings"
          anchors.left: parent.left;
          anchors.verticalCenter: parent.verticalCenter
          anchors.leftMargin: 20
          font.bold: true
        }
      }

      Flickable
      {
        id: flickable3
        flickableDirection: Flickable.VerticalFlick
        contentHeight: column3.height
        height: parent.height-configTop.height
        width: parent.width
        z: 0

        Column
        {
          id: column3
          anchors.horizontalCenter: parent.horizontalCenter
          spacing: 5
          width: parent.width-30

          Label
          {
            text: "Video Settings"
            width: parent.width
            height: font.pixelSize+32
            font.pixelSize: 28
            font.bold: true
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
          }

          CheckBox
          {
            id:    chkSaveCPU
            text:  "Pause application when minimized"
            width: parent.width
            checked: emulib.effSaveCPU
            onClicked: emulib.effSaveCPU=checked
          }

          CheckBox
          {
            id:    chkSync
            text:  "Limit frame rate"
            width: parent.width
            checked: emulib.effSync
            onClicked: emulib.effSync=checked
          }

          CheckBox
          {
            id:    chkSoften
            text:  "Soften display"
            width: parent.width
            checked: emulib.effSoften
            onClicked: emulib.effSoften=checked
          }

          CheckBox
          {
            id:    chkStretch
            text:  "Stretch display"
            width: parent.width
            checked: emulib.effStretch
            onClicked: emulib.effStretch=checked
          }

          CheckBox
          {
            id:    chkTVLines
            text:  "Simulate TV scanlines"
            width: parent.width
            checked: emulib.effTVLines
            onClicked: emulib.effTVLines=checked
          }

          CheckBox
          {
            id:    chkShowFPS
            text:  "Show framerate"
            width: parent.width
            checked: emulib.effShowFPS
            onClicked: emulib.effShowFPS=checked
          }

          CheckBox
          {
            id:    chkPenCues
            text:  "Show joypad cues"
            width: parent.width
            checked: emulib.effPenCues
            onClicked: emulib.effPenCues=checked
          }

          CheckBox
          {
            id:    chkVKbd
            text:  "Virtual keyboard"
            width: parent.width
            checked: emulib.effVKbd
            onClicked: emulib.effVKbd=checked
          }

          Label
          {
            text: "Audio Settings"
            width: parent.width
            height: font.pixelSize+32
            font.pixelSize: 28
            font.bold: true
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
          }

          Button
          {
            id:    btnSndRate
            text:  "Audio sampling rate " + dlgSndRate.model.get(dlgSndRate.selectedIndex).name
            width: parent.width
            onClicked:
            { 
              dlgSndRate.open();
              emulib.audioRate=dlgSndRate.selectedIndex;
            }
          }

          Button
          {
            id:    btnSndLatency
            text:  "Audio latency " + dlgSndLatency.model.get(dlgSndLatency.selectedIndex).name
            width: parent.width
            onClicked:
            {
              dlgSndLatency.open();
              emulib.audioLatency=dlgSndLatency.selectedIndex;
            }
          }
        }
      }
    }

    SelectionDialog
    {
      id: dlgSndRate
      titleText: "Audio Sampling Rate"
      selectedIndex: emulib.audioRate
      model: ListModel
      {
        ListElement { name: "8192Hz";  value: 8192;  }
        ListElement { name: "11025Hz"; value: 11025; }
        ListElement { name: "16000Hz"; value: 16000; }
        ListElement { name: "22050Hz"; value: 22050; }
        ListElement { name: "44100Hz"; value: 44100; }
      }
    }

    SelectionDialog
    {
      id: dlgSndLatency
      titleText: "Audio Latency"
      selectedIndex: emulib.audioLatency
      model: ListModel
      {
        ListElement { name: "50ms";  value: 50;  }
        ListElement { name: "100ms"; value: 100; }
        ListElement { name: "150ms"; value: 150; }
        ListElement { name: "200ms"; value: 200; }
        ListElement { name: "250ms"; value: 250; }
      }
    }
  }

  //
  // This is the Open File page
  //
  Page
  {
    id: filePage
    anchors.margins: 0
    tools: commonTools

    FolderListModel
    {
      id: folderList
      nameFilters: [ "*" ]
      folder: emulib.folder
    }

    Column
    {
      id: column4
      anchors.horizontalCenter: parent.horizontalCenter
      spacing: 5
      width: parent.width
      height: parent.height

      Rectangle
      {
        id: fileTop
        height: 64
        width: parent.width
        color: "#40A0FF"
        z: 10

        Label
        {
          text: "Select "+emulib.title+" File"
          anchors.left: parent.left
          anchors.verticalCenter: parent.verticalCenter
          anchors.leftMargin: 20
          font.bold: true
        }

        Image
        {
          source: "image://theme/icon-m-toolbar-up"
          anchors.rightMargin: 20
          anchors.right: parent.right;
          anchors.verticalCenter: parent.verticalCenter
          z: 20

          MouseArea
          {
            anchors.fill: parent
            onClicked: folderList.folder=folderList.parentFolder
          }
        }
      }

      ListView
      {
        id: viewFiles
        width: parent.width
        model: folderList
        height: parent.height-fileTop.height
        z: 0

        delegate: Item
        {
          height: 72
          width: parent.width

          Image
          {
            id:           imgIcon
            source:       "image://theme/icon-m-content-document"
            anchors.left: parent.left;
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: 10
          }

          Image
          {
            id:            imgArrow
            source:        "image://theme/icon-m-common-drilldown-arrow"
            visible:       folderList.isFolder(index)
            anchors.right: parent.right;
            anchors.verticalCenter: parent.verticalCenter
            anchors.rightMargin: 10
          }

          Label
          {
            text:          fileName
            anchors.left:  imgIcon.right;
            anchors.right: imgArrow.left;
            anchors.leftMargin:     16
            anchors.rightMargin:    16
            anchors.verticalCenter: parent.verticalCenter
          }

          MouseArea
          {
            anchors.fill: parent;
            onClicked:
            {
              if(folderList.isFolder(index))
              {
                folderList.folder = filePath;
                emulib.folder     = filePath;
              }
              else
              {
                emulib.command  = "open";
                emulib.fileName = filePath;
                Qt.quit();
              }
            }
          }
        }
      }
    }
  }

  //
  // This is the About page
  //
  Page
  {
    id: aboutPage
    anchors.margins: 0
    tools: commonTools

    Column
    {
      anchors.horizontalCenter: parent.horizontalCenter
      spacing: 5
      width: parent.width
      height: parent.height

      Rectangle
      {
        id: aboutTop
        height: 64
        width: parent.width
        color: "#40A0FF"
        z: 10

        Label
        {
          text: "About "+emulib.title
          anchors.left: parent.left;
          anchors.verticalCenter: parent.verticalCenter
          anchors.leftMargin: 20
          font.bold: true
        }
      }

      Flickable
      {
        flickableDirection: Flickable.VerticalFlick
        contentHeight: aboutColumn.height
        height: parent.height-configTop.height
        width: parent.width
        z: 0
  
        Column
        {
          id: aboutColumn
          anchors.horizontalCenter: parent.horizontalCenter
          spacing: 5
          width: parent.width-30
  
          Label
          {
            text: emulib.title
            width: parent.width
            height: font.pixelSize+16
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignBottom
            font.bold: true
            font.pixelSize: 50
            color: "#800000"
          }
  
          Label
          {
            text: "by "+emulib.author
            width: parent.width
            height: font.pixelSize+16
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            font.bold: true
            font.pixelSize: 24
          }
  
          Label
          {
            text: emulib.about
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: 24
          }
  
          Label
          {
            text: emulib.url
            width: parent.width
            height: font.pixelSize+16
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: 24
          }
  
          Button
          { 
            id: btnAboutOk
            width: parent.width
            text:  "Ok"
            onClicked: Qt.quit()
          }
        }
      }
    }
  }

  //
  // This is the Message page
  //
  Page
  {
    id: messagePage
    anchors.margins: 0

    Item
    {
      anchors.fill: parent

      Rectangle
      {
        id: messageTop
        height: 64
        width: parent.width
        color: "#40A0FF"
        anchors.top: parent.top
        z: 10

        Label
        {
          text: "Message from "+emulib.title
          anchors.left: parent.left;
          anchors.verticalCenter: parent.verticalCenter
          anchors.leftMargin: 20
          font.bold: true
        }
      }

      Item
      {
        anchors.top: messageTop.bottom
        anchors.bottom: parent.bottom
        width: parent.width

        Item
        {
          anchors.horizontalCenter: parent.horizontalCenter
          anchors.verticalCenter: parent.verticalCenter
          width: 3*parent.width/4
          height: 3*parent.height/4
  
          Label
          {
            text: emulib.message
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: 24
            anchors.top: parent.top
            anchors.bottom: btnMessageOk.top
          }
  
          Button
          { 
            id: btnMessageOk
            width: parent.width
            text:  "Ok"
            onClicked: Qt.quit()
            anchors.bottom: parent.bottom
          }
        }
      }
    }
  }
}
